<?php

class MyModel {
    public string $name;
    public array $classes = [];

    function __construct($name) {
        $this->name = preg_replace('/\..*$/','', $name);
    }

    static function readModelFile($file) {
        $model = new MyModel($file);
	$lines = file($file, FILE_IGNORE_NEW_LINES);
        foreach ($lines as $line) {
            if (preg_match('#<class:\s*(\w+)\s*\((.*)\)\s*>#',$line, $match)) {
                $class_name = $match[1];
                $att_list = preg_split("#\s*,\s*#", $match[2]);
#                print "read $class_name(".join(" ", $att_list).")\n";
                $c = new MyClass($class_name);
                foreach ($att_list as $a) {
                    $c->addAttribute($a);		
                }
                $model->classes[] = $c;
            } else {
                if (trim($line))
                  throw new Exception("illegal line '$line' in file '$file' detected");	
            }
        }
        return $model;
    }

    function __toString() {
        $str = "Model '$this->name'\n";
	foreach ($this->classes as $class)
	    $str .= "  $class";
        return $str;
    }
}

class MyClass {
    public string $name;
    public array $attributes = [];

    function __construct($name) {
        $this->name =  $name;
    }

    function addAttribute($definition) {
        $def = explode(':', $definition);
	if (count($def) < 2)
	    $def[] = "text";
        $this->attributes[] = new MyAttribute($def[0], $def[1]);
    }

    function __toString() {
        $str = "class $this->name\n";
        foreach ($this->attributes as $a)
           $str .= "    $a->name $a->type\n";
        return $str;
    }
}

class MyAttribute {
    public string $name;
    public string $type;

    static $types = ['text','integer','float'];

    function __construct($name, $type) {
        $this->name =  $name;
        $this->type = $type;
        if (! $this->has_primitive_type())
	    die("Attribute '$this->name' has an unknown type ($type). Valid types are: ".join(", ", self::$types));
    }

    function has_primitive_type() {
        return in_array($this->type, self::$types);
    }
}


